@@ -28,7 +28,8 @@
  * along with OpenShot Library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "../../include/effects/Deinterlace.h"
+#include "Deinterlace.h"
+#include "Exceptions.h"
 
 using namespace openshot;
 
@@ -62,7 +63,7 @@ void Deinterlace::init_effect_details()
 
 // This method is required for all derived classes of EffectBase, and returns a
 // modified openshot::Frame object
-std::shared_ptr<Frame> Deinterlace::GetFrame(std::shared_ptr<Frame> frame, int64_t frame_number)
+std::shared_ptr<openshot::Frame> Deinterlace::GetFrame(std::shared_ptr<openshot::Frame> frame, int64_t frame_number)
 {
 	// Get original size of frame's image
 	int original_width = frame->GetImage()->width();
@@ -73,7 +74,7 @@ std::shared_ptr<Frame> Deinterlace::GetFrame(std::shared_ptr<Frame> frame, int64
 	const unsigned char* pixels = image->bits();
 
 	// Create a smaller, new image
-	QImage deinterlaced_image(image->width(), image->height() / 2, QImage::Format_RGBA8888);
+	QImage deinterlaced_image(image->width(), image->height() / 2, QImage::Format_RGBA8888_Premultiplied);
 	const unsigned char* deinterlaced_pixels = deinterlaced_image.bits();
 
 	// Loop through the scanlines of the image (even or odd)
@@ -86,7 +87,9 @@ std::shared_ptr<Frame> Deinterlace::GetFrame(std::shared_ptr<Frame> frame, int64
 	}
 
 	// Resize deinterlaced image back to original size, and update frame's image
-	image = std::shared_ptr<QImage>(new QImage(deinterlaced_image.scaled(original_width, original_height, Qt::IgnoreAspectRatio, Qt::FastTransformation)));
+	image = std::make_shared<QImage>(deinterlaced_image.scaled(
+		original_width, original_height,
+		Qt::IgnoreAspectRatio, Qt::FastTransformation));
 
 	// Update image on frame
 	frame->AddImage(image);
@@ -159,6 +162,9 @@ std::string Deinterlace::PropertiesJSON(int64_t requested_frame) const {
 	root["isOdd"]["choices"].append(add_property_choice_json("Yes", true, isOdd));
 	root["isOdd"]["choices"].append(add_property_choice_json("No", false, isOdd));
 
+	// Set the parent effect which properties this effect will inherit
+	root["parent_effect_id"] = add_property_json("Parent", 0.0, "string", info.parent_effect_id, NULL, -1, -1, false, requested_frame);
+
 	// Return formatted string
 	return root.toStyledString();
 }
